unit uCcw; { mk, 17.2.2004, Sedgewick, Algorithmen, S. 402/403 }

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  TZustand = (z0,z1,z2);            // selbst definierter Datentyp

  TPunkt = record                   // record ist wie class ,nur
             x,y : real             // ohne Methoden, ohne private
           end;

  TStreckenzug = class(TObject)
  private
    p   : array[0..2] of TPunkt;
    ccw : integer;
  public
    procedure setP(nr : integer; x,y : real);
    function getP(nr : integer) : TPunkt;
    function getCcw : integer;
  end;

  TForm1 = class(TForm)
    bZeichne: TButton;
    eP0x: TEdit; lP0x: TLabel; eP0y: TEdit; lP0y: TLabel;
    eP1x: TEdit; lP1x: TLabel; eP1y: TEdit; lP1y: TLabel;
    eP2x: TEdit; lP2x: TLabel; eP2y: TEdit; lP2y: TLabel;
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure bZeichneClick(Sender: TObject);
  private
    zustand     : TZustand;
    Streckenzug : TStreckenzug;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation
{$R *.dfm}

procedure TStreckenzug.setP(nr : integer; x,y : real);
begin
  p[nr].x := x;
  p[nr].y := y;
end;

function TStreckenzug.getP(nr : integer) : TPunkt;
begin
  result.x := p[nr].x;
  result.y := p[nr].y;
end;

function TStreckenzug.getCcw : integer;

  function ccw(p0,p1,p2 : TPunkt) : integer;
  var
    dx1,dx2,dy1,dy2 : real;
  begin
    dx1 := p1.x - p0.x; dy1 := p1.y - p0.y;
    dx2 := p2.x - p0.x; dy2 := p2.y - p0.y;
    if dx1*dy2 > dy1*dx2 then result := +1;
    if dx1*dy2 < dy1*dx2 then result := -1;
    if dx1*dy2 = dy1*dx2 then
    begin
      if (dx1*dx2 < 0) or (dy1*dy2 < 0) then
      result := -1
      else
        if (dx1*dx1 + dy1*dy1) >= (dx2*dx2 + dy2*dy2) then
          result := 0
        else
          result := 1;
    end;
  end;

begin
  result := ccw(p[0],p[1],p[2]);
end;



procedure TForm1.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  case zustand of                                         
    z0 : begin
           Streckenzug.setP(0,x,y);
           zustand := z1;
           canvas.Pixels[x,y] := clRed;
           canvas.MoveTo(x,y);
         end;
    z1 : begin
           Streckenzug.setP(1,x,y);
           zustand := z2;
           canvas.Pen.Color := clRed;
           //canvas.Pen.Width := 2;
           canvas.LineTo(x,y);
         end;
    z2 : begin
           Streckenzug.setP(2,x,y);
           zustand := z0;
           canvas.Pen.Color := clGreen;
           //canvas.Pen.Width := 2;
           canvas.LineTo(x,y);
           canvas.TextOut(x,y,IntToStr(Streckenzug.getCcw));
         end;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Streckenzug := TStreckenzug.Create;
  zustand := z0;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Streckenzug.Free;
end;

procedure TForm1.bZeichneClick(Sender: TObject);
var
  p0x,p0y,p1x,p1y,p2x,p2y : real;
begin
  p0x := StrToFloat(eP0x.Text); p0y := StrToFloat(eP0y.Text);
  p1x := StrToFloat(eP1x.Text); p1y := StrToFloat(eP1y.Text);
  p2x := StrToFloat(eP2x.Text); p2y := StrToFloat(eP2y.Text);

  Streckenzug.setP(0,p0x,p0y);
  Streckenzug.setP(1,p1x,p1y);
  Streckenzug.setP(2,p2x,p2y);
  canvas.MoveTo(Round(p0x),Round(p0y));
  canvas.Pen.Color := clRed;
  //canvas.Pen.Width := 2;
  canvas.LineTo(Round(p1x),Round(p1y));
  canvas.Pen.Color := clGreen;
  //canvas.Pen.Width := 2;
  canvas.LineTo(Round(p2x),Round(p2y));
  canvas.TextOut(Round(p2x),Round(p2y),IntToStr(Streckenzug.getCcw));
end;

end.
